// RobotView.cpp : implementation of the CRobotView class
//

#include "stdafx.h"
#include "Robot.h"

#include "RobotDoc.h"
#include "RobotView.h"
#include "griddlg.h"
#include "robotposdlg.h"
#include "movedlg.h"
#include "exitposdlg.h"
#include "statementdlg.h"


#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif


typedef struct
{
	BOOL l,t,r,b, x;
} FOURWAY;

FOURWAY fw[20][20];
CDC robot, exitgate, xxx;

char *robot_dir_str[4] = {"East", "North", "West", "South"};
int dir_map[4] = {0, 3, 1, 2};
int robot_dir = 1;

int X =	6;
int	Y =	6;
int robot_x = 0;
int robot_y = 2;
int exit_x = 5;
int exit_y = 2;

int test_x = 0;
int test_y = 0;
int test_cnt = 0;

CString states[50];
int states_idx = 0;

BOOL run = TRUE;

/////////////////////////////////////////////////////////////////////////////
// CRobotView

IMPLEMENT_DYNCREATE(CRobotView, CView)

BEGIN_MESSAGE_MAP(CRobotView, CView)
	ON_MESSAGE(WM_STATEMENT, OnStateProcess)
	ON_MESSAGE(WM_EXECUTEMSG, ExecuteCmd)
	//{{AFX_MSG_MAP(CRobotView)
	ON_WM_LBUTTONDOWN()
	ON_COMMAND(IDM_GRIDSIZE, OnGridsize)
	ON_COMMAND(IDM_ROBOTPOS, OnRobotpos)
	ON_COMMAND(IDM_ROBOTCMD, OnRobotcmd)
	ON_WM_DESTROY()
	ON_COMMAND(IDM_EXITPOS, OnExitpos)
	ON_COMMAND(IDM_STATEMENT, OnStatement)
	ON_COMMAND(IDM_RUN, OnRun)
	ON_WM_TIMER()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CRobotView construction/destruction

CRobotView::CRobotView()
{
	// TODO: add construction code here

}

CRobotView::~CRobotView()
{
}

BOOL CRobotView::PreCreateWindow(CREATESTRUCT& cs)
{
	// TODO: Modify the Window class or styles here by modifying
	//  the CREATESTRUCT cs

	return CView::PreCreateWindow(cs);
}

/////////////////////////////////////////////////////////////////////////////
// CRobotView drawing

void CRobotView::OnDraw(CDC* pDC)
{
	CRobotDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	DrawGrid();
	DrawWall();
	DrawRobotAndExit();
}

/////////////////////////////////////////////////////////////////////////////
// CRobotView diagnostics

#ifdef _DEBUG
void CRobotView::AssertValid() const
{
	CView::AssertValid();
}

void CRobotView::Dump(CDumpContext& dc) const
{
	CView::Dump(dc);
}

CRobotDoc* CRobotView::GetDocument() // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CRobotDoc)));
	return (CRobotDoc*)m_pDocument;
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CRobotView message handlers

void CRobotView::OnInitialUpdate() 
{
	CView::OnInitialUpdate();
	
	ZeroMemory((void*)fw, sizeof(fw));

	CBitmap b1, b2, b3;

	CClientDC dc(this);
	b1.LoadBitmap(IDB_ROBOT);
	robot.CreateCompatibleDC(&dc);
	robot.SelectObject(&b1);

	b2.LoadBitmap(IDB_EXIT);
	exitgate.CreateCompatibleDC(&dc);
	exitgate.SelectObject(&b2);

	b3.LoadBitmap(IDB_X);
	xxx.CreateCompatibleDC(&dc);
	xxx.SelectObject(&b3);

	moveDlg = new CMoveDlg(this);
	stateDlg = new CStatementDlg(this);

	int i;

	for (i=0; i<50; i++)
		states[i].Format("%02d: ", i);
}

void CRobotView::DrawGrid()
{
	int i;

	CClientDC dc(this);
	CPen newPen(PS_SOLID, 2, RGB(200, 200, 200));
	RECT r;
	WORD width, height;

	GetClientRect(&r);
	height = r.bottom / Y;
	width = r.right / X;

	dc.SelectObject(&newPen);

	for (i=1; i<Y; i++)
	{
		dc.MoveTo(r.left, (i*height));
		dc.LineTo(r.right, (i*height));
	}

	for (i=1; i<X; i++)
	{
		dc.MoveTo((i*width), r.top);
		dc.LineTo((i*width), r.bottom);
	}
}

void CRobotView::DrawWall()
{
	int i,j;

	CClientDC dc(this);
	CPen newPen(PS_SOLID, 2, RGB(60, 60, 60));
	RECT r;
	WORD width, height;


	GetClientRect(&r);
	height = r.bottom / Y;
	width = r.right / X;

	dc.SelectObject(&newPen);

	for (i=0; i<X; i++)
		for (j=0; j<Y; j++)
		{
			// draw X
			if (!run && fw[j][i].x)
				dc.StretchBlt(width * i + 1, height * j + 1, 
						width-5, height-5, &xxx, 0, 0, 47, 42, SRCCOPY);

			// left
			if (fw[j][i].l && i > 0)
			{
				dc.MoveTo(width*i, height*j);
				dc.LineTo(width*i, height*(j+1));
			}

			// right
			if (fw[j][i].r && i < (X-1))
			{
				dc.MoveTo(width*(i+1), height*j);
				dc.LineTo(width*(i+1), height*(j+1));
			}

			// top
			if (fw[j][i].t && j > 0)
			{
				dc.MoveTo(width*i, height*j);
				dc.LineTo(width*(i+1), height*j);
			}

			// bottom
			if (fw[j][i].b && j < (Y-1))
			{
				dc.MoveTo(width*i, height*(j+1));
				dc.LineTo(width*(i+1), height*(j+1));
			}
		}
}

void CRobotView::OnLButtonDown(UINT nFlags, CPoint point) 
{
	RECT r;
	WORD blk_x, blk_y;
	WORD width, height;
	WORD e,w,s,n, dir, sel;

	GetClientRect(&r);
	height = r.bottom / Y;
	width = r.right / X;

	blk_x = point.x / width;
	blk_y = point.y / height;
	
	// east, west, south, north

	dir = 0; // east by default

	// calculate distance
	e = width*(blk_x+1) - point.x;
	w = point.x - width*blk_x;
	s = height*(blk_y+1) - point.y;
	n = point.y - height*blk_y;

	sel = e;

	if (w < sel)
	{
		dir = 1;
		sel = w;
	}

	if (s < sel)
	{
		dir = 2;
		sel = s;
	}

	if (n < sel)
	{
		dir = 3;
		sel = n;
	}

	// east
	if (dir == 0)
	{
		fw[blk_y][blk_x].r = !fw[blk_y][blk_x].r;
		if (blk_x < (X-1))
			fw[blk_y][blk_x+1].l = fw[blk_y][blk_x].r;
	}

	// west
	if (dir == 1)
	{
		fw[blk_y][blk_x].l = !fw[blk_y][blk_x].l;
		if (blk_x > 0)
			fw[blk_y][blk_x-1].r = fw[blk_y][blk_x].l;
	}

	// south
	if (dir == 2)
	{
		fw[blk_y][blk_x].b = !fw[blk_y][blk_x].b;
		if (blk_y < (Y-1))
			fw[blk_y+1][blk_x].t = fw[blk_y][blk_x].b;
	}

	// north
	if (dir == 3)
	{
		fw[blk_y][blk_x].t = !fw[blk_y][blk_x].t;
		if (blk_y > 0)
			fw[blk_y-1][blk_x].b = fw[blk_y][blk_x].t;
	}

	CView::OnLButtonDown(nFlags, point);

	Invalidate();
}

void CRobotView::OnGridsize() 
{
	CGridDlg dlg;

	dlg.m_grid_x = X;
	dlg.m_grid_y = Y;

	UpdateData(FALSE);
	
	if (dlg.DoModal() == IDOK)
	{
		UpdateData(TRUE);

		X = dlg.m_grid_x;
		Y = dlg.m_grid_y;

		if (robot_x > (X-1))
			robot_x = X-1;

		if (robot_y > (Y-1))
			robot_y = Y-1;

		Invalidate();
	}
}

void CRobotView::DrawRobotAndExit()
{
	RECT r;
	WORD width, height;
	CClientDC dc(this);

	GetClientRect(&r);
	height = r.bottom / Y;
	width = r.right / X;


	dc.StretchBlt(width * robot_x + 1, height * robot_y + 1, 
		width-5, height-5, &robot, 0, 0,
		31, 52, SRCCOPY);

	dc.StretchBlt(width * exit_x + 1, height * exit_y + 1, 
		width-5, height-5, &exitgate, 0, 0,
		48, 48, SRCCOPY);

	POINT p[3];

	// east
	if (dir_map[robot_dir] == 0)
	{
		p[0].x = width * (robot_x+1) - width/5;
		p[0].y = height * robot_y;
		p[1].x = width * (robot_x+1) - width/5;
		p[1].y = height * (robot_y+1);
		p[2].x = width * (robot_x+1);
		p[2].y = height * (robot_y) + height/2;
	}
		
	// west
	if (dir_map[robot_dir] == 1)
	{
		p[0].x = width * robot_x + width/5;
		p[0].y = height * robot_y;
		p[1].x = width * robot_x + width/5;
		p[1].y = height * (robot_y+1);
		p[2].x = width * robot_x;
		p[2].y = height * (robot_y) + height/2;
	}

	// south
	if (dir_map[robot_dir] == 2)
	{
		p[0].x = width * (robot_x+1);
		p[0].y = height * (robot_y+1) - height/5;
		p[1].x = width * robot_x;
		p[1].y = height * (robot_y+1) - height/5;
		p[2].x = width * robot_x + width/2;
		p[2].y = height * (robot_y+1);
	}

	// north
	if (dir_map[robot_dir] == 3)
	{
		p[0].x = width * (robot_x+1);
		p[0].y = height * robot_y + height/5;
		p[1].x = width * robot_x;
		p[1].y = height * robot_y + height/5;
		p[2].x = width * robot_x + width/2;
		p[2].y = height * robot_y;
	}

	dc.Polygon(p, 3);
}

void CRobotView::OnRobotpos() 
{
	CRobotPosDlg dlg;

	dlg.m_pos_x = robot_x;
	dlg.m_pos_y = robot_y;

	UpdateData(FALSE);

	if (dlg.DoModal() == IDOK)
	{
		UpdateData(TRUE);

		robot_x = dlg.m_pos_x;
		robot_y = dlg.m_pos_y;

		if (robot_x > (X-1))
			robot_x = X-1;

		if (robot_y > (Y-1))
			robot_y = Y-1;

		Invalidate();
	}
}

void CRobotView::OnRobotcmd() 
{
	moveDlg->ShowWindow(SW_SHOW);
}

void CRobotView::OnDestroy() 
{
	CView::OnDestroy();
	
	delete moveDlg;	
}

void CRobotView::ExecuteCmd(WPARAM w, LPARAM l)
{
	BOOL bCollision = FALSE;

	// left
	if (w == 0)
	{
		robot_dir++;

		if (robot_dir > 3)
			robot_dir = 0;

		Invalidate();
	}

	// right
	if (w == 1)
	{
		robot_dir--;

		if (robot_dir < 0)
			robot_dir = 3;

		Invalidate();
	}
	
	// move
	if (w == 2)
	{
		// east
		if (dir_map[robot_dir] == 0)
			if (fw[robot_y][robot_x].r || (robot_x+1) > (X-1))
				bCollision = TRUE;
			else
				robot_x++;

		// west
		if (dir_map[robot_dir] == 1)
			if (fw[robot_y][robot_x].l || (robot_x-1) < 0)
				bCollision = TRUE;
			else
				robot_x--;

		// south
		if (dir_map[robot_dir] == 2)
			if (fw[robot_y][robot_x].b || (robot_y+1) > (Y-1))
				bCollision = TRUE;
			else
				robot_y++;

		// north
		if (dir_map[robot_dir] == 3)
			if (fw[robot_y][robot_x].t || (robot_y-1) < 0)
				bCollision = TRUE;
			else
				robot_y--;

		if (bCollision)
		{
//			KillTimer(1234);
//			::MessageBox(NULL, "   ϴ!", "Robot", MB_ICONSTOP);
		}
		else
			Invalidate();
	}
}

void CRobotView::OnExitpos() 
{
	CExitPosDlg dlg;
	
	dlg.m_exit_x = exit_x;
	dlg.m_exit_y = exit_y;

	UpdateData(FALSE);

	if (dlg.DoModal() == IDOK)
	{
		UpdateData(TRUE);

		exit_x = dlg.m_exit_x;
		exit_y = dlg.m_exit_y;

		if (exit_x > (X-1))
			exit_x = X-1;

		if (exit_y > (Y-1))
			exit_y = Y-1;

		Invalidate();
	}
}

void CRobotView::OnStatement() 
{
	int i;

	for (i=0; i<50; i++)
		stateDlg->states[i] = states[i];

	stateDlg->Retrieve();
	stateDlg->ShowWindow(SW_SHOW);
	stateDlg->ShowIndicator();
}

void CRobotView::OnStateProcess(WPARAM w, LPARAM l)
{
	int i, j;

	// done
	if (w == 0)
	{
		for (i=0; i<50; i++)
			states[i] = stateDlg->states[i];
	}

	// stop
	if (w == 1)
	{
		KillTimer(1234);
		states_idx = 0;
		stateDlg->m_list.SetCurSel(states_idx);
	}

	// run
	if (w == 2)
	{
		states_idx = 0;
		stateDlg->m_list.SetCurSel(states_idx);
		OnRun();
	}

	// test all
	if (w == 3)
	{
		run = FALSE;
		stateDlg->ResetIndicator();

		for (i=0; i<X; i++)
			for (j=0; j<Y; j++)
				fw[j][i].x = FALSE;

		if (stateDlg->IsRanDir())
			robot_dir = (rand() % 4);
		else
			robot_dir = 1;

		test_x = 0;
		test_y = 0;

		robot_x = test_x;
		robot_y = test_y;

		test_cnt = 0;

		SetTimer(1234, 30, NULL);
	}

	// reset
	if (w == 4)
	{
		for (i=0; i<X; i++)
			for (j=0; j<Y; j++)
				fw[j][i].x = FALSE;

		stateDlg->ResetIndicator();

		Invalidate();
	}
}

void CRobotView::OnRun() 
{
	run = TRUE;

	if (stateDlg->IsRanDir())
		robot_dir = (rand() % 4);
	else
		robot_dir = 1;

	stateDlg->ResetIndicator();
	SetTimer(1234, 500, NULL);
}

void CRobotView::OnTimer(UINT nIDEvent) 
{
	if (nIDEvent == 1234)
	{
		if (states[states_idx].Find("MOVE",0) != -1)
		{
			ExecuteCmd(2, 0); // move
			stateDlg->NextIndicator();
			states_idx++;
		}
		else
		if (states[states_idx].Find("LEFT", 0) != -1)
		{
			ExecuteCmd(0, 0); // left
			stateDlg->NextIndicator();
			states_idx++;
		}
		else
		if (states[states_idx].Find("RIGHT", 0) != -1)
		{
			ExecuteCmd(1, 0); // right
			stateDlg->NextIndicator();
			states_idx++;
		}
		else
		if (states[states_idx].Find("IF WALL THEN GOTO", 0) != -1 ||
			states[states_idx].Find("WALL", 0) != -1)
		{
			BOOL bCol = FALSE;

			// east
			if (dir_map[robot_dir] == 0)
				if (fw[robot_y][robot_x].r || (robot_x+1) > (X-1))
					bCol = TRUE;

			if (dir_map[robot_dir] == 1)
				if (fw[robot_y][robot_x].l || (robot_x-1) < 0)
					bCol = TRUE;
			
			if (dir_map[robot_dir] == 2)
				if (fw[robot_y][robot_x].b || (robot_y+1) > (Y-1))
					bCol = TRUE;

			if (dir_map[robot_dir] == 3)
				if (fw[robot_y][robot_x].t || (robot_y-1) < 0)
					bCol = TRUE;

			if (bCol)
			{
				if (states[states_idx].Find("IF WALL THEN GOTO", 0) != -1)
					states_idx = atoi(((LPCTSTR)states[states_idx])+21);
				else
					states_idx = atoi(((LPCTSTR)states[states_idx])+8);

				stateDlg->SetIndicator(states_idx);
			}
			else
			{
				stateDlg->NextIndicator();
				states_idx++;
			}

		}
		else
		if (states[states_idx].Find("GOTO", 0) != -1)
		{
			char buf[100];

			strcpy(buf, (LPCTSTR)states[states_idx]);
			states_idx = atoi(buf+8);
			stateDlg->SetIndicator(states_idx);
		}
		else
		if (states[states_idx].Find("STOP", 0) != -1)
		{
			KillTimer(1234);

			if (run && robot_x == exit_x && robot_y == exit_y)
				::MessageBox(NULL, "ǥ !", "Robot", MB_ICONASTERISK);

			if (robot_x != exit_x || robot_y != exit_y) 
				fw[test_y][test_x].x = TRUE;

			Invalidate();

			stateDlg->ResetIndicator();

			states_idx = 0;

			if (stateDlg->IsRanDir())
				robot_dir = (rand() % 4);
			else
				robot_dir = 1;
	
			test_cnt++;

			if (test_cnt != (X*Y))
			{
				if (test_x == (X-1))
				{
					test_x=0;
					test_y++;
				}
				else
					test_x++;

				robot_x = test_x;
				robot_y = test_y;

				if (!run)	// test all
					SetTimer(1234, 30, NULL);
			}
			else
			{
				test_x = 0;
				test_y = 0;
				test_cnt = 0;

				MessageBox("End Test");
			}
		}
		else
		{
			stateDlg->NextIndicator();
			states_idx++;
		}
	}
	
	CView::OnTimer(nIDEvent);
}
